@@ -891,7 +891,7 @@ brl_characters (void          *ctx,
 	    gchar  *str_utf = NULL;
 	    gchar  *str_crt = NULL;
 	    glong  str_len;
-	    str_utf = g_strndup (ch, len);
+	    str_utf = g_strndup ((const gchar*)ch, len);
 	    str_len = g_utf8_strlen (str_utf, -1);
 	    /* convert text to braille dots */
 	    dotbuff = malloc (str_len);			
