@@ -32,6 +32,7 @@
 #include "srspc.h"
 #include "srbrl.h"
 
+#include "libke.h"
 #include "srmain.h" 
 #include "srpres.h"
 #include "screen-review.h"
@@ -1633,23 +1634,22 @@ src_nav_statusbar ()
 }
 
 
-extern gboolean SPI_generateMouseEvent (long int x, long int y, gchar *name);
-
 static gboolean
-src_generateMouseEvent (long int x, long int y, gchar *name)
+src_mouse_button_action (KEMouseButton button, KEMouseButtonAction action)
 {
     if (src_xevie_present)
-	return SPI_generateMouseEvent (x, y, name);
+	return ke_mouse_button_action (button, action);
     gdk_beep ();
     return FALSE;
 }
 
+
 static gboolean
 src_mouse_left_press ()
 {
     if (src_button_left_pressed) return FALSE;
     src_button_left_pressed = !src_button_left_pressed;
-    return src_generateMouseEvent (-1, -1, "b1p");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_LEFT, KE_MOUSE_BUTTON_PRESS);
 }
 
 static gboolean
@@ -1657,21 +1657,21 @@ src_mouse_left_release ()
 {
     if (!src_button_left_pressed) return FALSE;
     src_button_left_pressed = !src_button_left_pressed;
-    return src_generateMouseEvent (-1, -1, "b1r");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_LEFT, KE_MOUSE_BUTTON_RELEASE);
 }
 
 static gboolean
 src_mouse_left_click ()
 {
     if (src_button_left_pressed) return FALSE;
-    return src_generateMouseEvent (-1, -1, "b1c");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_LEFT, KE_MOUSE_BUTTON_CLICK);
 }
 
 /*
 static gboolean
 src_mouse_left_double_click ()
 {
-    return src_generateMouseEvent (-1, -1, "b1d");
+	return src_mouse_button_action (KE_MOUSE_BUTTON_LEFT, KE_MOUSE_BUTTON_DOUBLECLICK);
 }
 */
 
@@ -1680,7 +1680,7 @@ src_mouse_middle_press ()
 {
     if (src_button_middle_pressed) return FALSE;
     src_button_middle_pressed = !src_button_middle_pressed;
-    return src_generateMouseEvent (-1, -1, "b3p");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_MIDDLE, KE_MOUSE_BUTTON_PRESS);
 }
 
 static gboolean
@@ -1688,21 +1688,21 @@ src_mouse_middle_release ()
 {
     if (!src_button_middle_pressed) return FALSE;
     src_button_middle_pressed = !src_button_middle_pressed;
-    return src_generateMouseEvent (-1, -1, "b3r");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_MIDDLE, KE_MOUSE_BUTTON_RELEASE);
 }
 
 static gboolean
 src_mouse_middle_click ()
 {
     if (src_button_middle_pressed) return FALSE;
-    return src_generateMouseEvent (-1, -1, "b3c");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_MIDDLE, KE_MOUSE_BUTTON_CLICK);
 }
 
 /*
 static gboolean
 src_mouse_middle_double_click ()
 {
-    return src_generateMouseEvent (-1, -1, "b3d");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_MIDDLE, KE_MOUSE_BUTTON_DOUBLECLICK);
 }
 */
 
@@ -1712,7 +1712,7 @@ src_mouse_right_press ()
 {
     if (src_button_right_pressed) return FALSE;
     src_button_right_pressed = !src_button_right_pressed;
-    return src_generateMouseEvent (-1, -1, "b2p");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_RIGHT, KE_MOUSE_BUTTON_PRESS);
 }
 
 static gboolean
@@ -1720,21 +1720,21 @@ src_mouse_right_release ()
 {
     if (!src_button_right_pressed) return FALSE;
     src_button_right_pressed = !src_button_right_pressed;
-    return src_generateMouseEvent (-1, -1, "b2r");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_RIGHT, KE_MOUSE_BUTTON_RELEASE);
 }
 
 static gboolean
 src_mouse_right_click ()
 {
     if (src_button_right_pressed) return FALSE;
-    return src_generateMouseEvent (-1, -1, "b2c");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_RIGHT, KE_MOUSE_BUTTON_CLICK);
 }
 
 /*
 static gboolean
 src_mouse_right_double_click ()
 {
-    return src_generateMouseEvent (-1, -1, "b2d");
+    return src_mouse_button_action (KE_MOUSE_BUTTON_RIGHT, KE_MOUSE_BUTTON_DOUBLECLICK);
 }
 */
 
@@ -1793,7 +1793,7 @@ src_mouse_goto_crt_sro ()
     obj = src_crt_sro;
     sro_add_reference (obj);
     if (src_get_mouse_coordinates_from_sro (src_crt_sro, &x, &y, TRUE))
-	rv = SPI_generateMouseEvent (x, y, "abs");
+	rv = ke_mouse_move (x, y);
     sro_release_reference (obj);
     return rv;
 }
@@ -2656,11 +2656,8 @@ src_flat_review_line (gint index)
     }
 	
     line = screen_review_get_line (index, &y1, &y2);
-	
-    flat_review_y = (y1 + y2) /2;
-    SPI_generateMouseEvent (flat_review_x,
-			    flat_review_y,
-			    "abs");
+    flat_review_y = (y1 + y2) / 2;
+    ke_mouse_move (flat_review_x, flat_review_y);
     if ((!line || line->is_empty) && src_use_speech)
     {
 	if (line->is_empty == 1)
@@ -2712,7 +2709,7 @@ src_flat_review_line (gint index)
     }
     else if (line && (src_use_braille || src_use_braille_monitor))
 	src_present_line_to_braille_or_brlmon (line);
-	
+
     last_line = index;
 
     return TRUE;
