@@ -1,7 +1,7 @@
 /* SRLow.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -60,7 +60,6 @@
 #define SRL_EVENT_NAME_WINDOW_RESTORE		"window:restore"
 #define SRL_EVENT_NAME_WINDOW_ACTIVATE		"window:activate"
 #define SRL_EVENT_NAME_WINDOW_DEACTIVATE	"window:deactivate"
-#define SRL_EVENT_NAME_MOUSE_ABS		"mouse:abs"
 #define SRL_EVENT_NAME_CONTENT_CHANGED		"object:property-changed:accessible-content"
 #define SRL_EVENT_NAME_WINDOW_SWITCH		"window:switch"
 #define SRL_EVENT_NAME_TOOLTIP_SHOW		"tooltip:show"
@@ -108,7 +107,6 @@ typedef enum _SRLEventType
     SRL_EVENT_WINDOW_ACTIVATE,
     SRL_EVENT_WINDOW_DEACTIVATE,
     SRL_EVENT_WINDOW_TITLELIZE,
-    SRL_EVENT_MOUSE_ABS,
     SRL_EVENT_TOOLTIP_SHOW,
     SRL_EVENT_TOOLTIP_HIDE,
     SRL_EVENT_ACTIVE_DESCENDANT_CHANGED,
@@ -124,7 +122,6 @@ typedef enum _SRLEventType
 
 typedef enum _SRLEventPriority
 {
-    SRL_EVENT_PRIORITY_MOUSE,
     SRL_EVENT_PRIORITY_TOOLTIP,
     SRL_EVENT_PRIORITY_FOCUS,
     SRL_EVENT_PRIORITY_WATCHED,
@@ -160,7 +157,6 @@ static SRLEventTypeEventName srl_events_type_name[] =
 	{SRL_EVENT_WINDOW_RESTORE,		SRL_EVENT_NAME_WINDOW_RESTORE},
 	{SRL_EVENT_WINDOW_ACTIVATE,		SRL_EVENT_NAME_WINDOW_ACTIVATE},
 	{SRL_EVENT_WINDOW_DEACTIVATE,		SRL_EVENT_NAME_WINDOW_DEACTIVATE},
-	{SRL_EVENT_MOUSE_ABS,			SRL_EVENT_NAME_MOUSE_ABS},
 	{SRL_EVENT_ACTIVE_DESCENDANT_CHANGED, 	SRL_EVENT_NAME_ACTIVE_DESCENDANT_CHANGED},
 	{SRL_EVENT_NAME_CHANGED,		SRL_EVENT_NAME_NAME_CHANGED},
 	{SRL_EVENT_VISIBLE_DATA_CHANGED,	SRL_EVENT_NAME_VISIBLE_DATA_CHANGED},
@@ -400,22 +396,6 @@ srl_log_gnopernicus_event_user_obj (SRLEvent *event,
     SR_freeString (type);
 }
 
-static void
-srl_log_gnopernicus_event_user_mouse (SRLEvent *event,
-				      SREvent *ev)
-{
-    SRPoint *point;
-
-    srl_assert (event && ev);
-
-    if (!srl_log_is_gnopernicus_event_logged ())
-	return;
-
-    sre_get_event_data (ev, (void**)&point);
-    fprintf (stderr, "\nGN:%xp--\"mouse:move\" "
-			"at %d, %d", SRL_PTR (event), point->x, point->y);
-}
-
 static G_CONST_RETURN Accessible* srle_get_acc (SRLEvent *event);
 static G_CONST_RETURN gchar* 	  srle_get_reason (SRLEvent *event);
 
@@ -497,7 +477,6 @@ srl_log_terminate ()
 #else /* SRL_LOG */
 #define srl_log_at_spi_event(X)
 #define srl_log_gnopernicus_event_user_obj(X, Y)
-#define srl_log_gnopernicus_event_user_mouse(X, Y)
 #define srl_log_important_event(X)
 #define srl_log_reentrancy_event(X)
 #define srl_log_init()
@@ -686,6 +665,71 @@ srl_get_focused_cell (Accessible *acc_table)
     return rv;
 }
 
+static Accessible*
+srl_get_focused_from_acc (Accessible *acc)
+{
+    Accessible *focused = NULL;
+    gint cnt, i;
+
+    srl_assert (acc);
+    
+    cnt = Accessible_getChildCount (acc);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (acc, i);
+	if (!child)
+	    continue;
+	if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
+	{
+	    focused = child;
+	    Accessible_ref (focused);
+	}
+	else
+	    focused = srl_get_focused_from_acc (child);
+	
+	Accessible_unref (child);
+	if (focused)
+	    break;
+    }
+
+    return focused;
+}
+
+/* function returns the active child in a tree. The search
+   is limited to maximum _limit_ number of children, children's chidren etc */
+static Accessible*
+srl_tree_get_active_item (Accessible *tree,
+			  gint *limit)
+{
+    Accessible *active = NULL;
+    gint cnt, i;
+
+    srl_assert (tree);
+
+    cnt = Accessible_getChildCount (tree);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (tree, i);
+	if (!child)
+	    continue;
+	(*limit)--;
+	if (srl_acc_has_state (child, SPI_STATE_ACTIVE))
+	{
+	    active = child;
+	    Accessible_ref (active);
+	}
+	else if (srl_acc_has_state (child, SPI_STATE_EXPANDED))
+	    active = srl_tree_get_active_item (child, limit);
+	Accessible_unref (child);
+	if (active || *limit < 0)
+	    break;
+    }
+    
+    return active;
+}
+
 static gboolean
 srle_acc_has_state (SRLEvent *event,
 		    AccessibleState state)
@@ -770,6 +814,19 @@ srle_set_acc (SRLEvent *event)
 	    AccessibleSelection_unref (selection);
 	}
     }
+    else if (srle_has_type (event, SRL_EVENT_FOCUS) &&
+	     srle_acc_has_role (event, SPI_ROLE_TREE))
+    {
+	gint limit = SRL_MAX_CHILDREN_CNT;
+	event->acc = srl_tree_get_active_item ((Accessible*)srle_get_acc (event), 
+						 &limit);
+    }
+    else if (srle_has_type (event, SRL_EVENT_FOCUS) &&
+		srle_acc_has_role (event, SPI_ROLE_TABLE) &&
+		!srle_acc_has_state (event, SPI_STATE_MANAGES_DESCENDANTS))
+    {
+	event->acc = srl_get_focused_from_acc ((Accessible*)srle_get_acc (event));
+    }
     else if (srle_has_type (event, SRL_EVENT_ACTIVE_DESCENDANT_CHANGED))
     {
 	event->acc = AccessibleActiveDescendantChangedEvent_getActiveDescendant (event->acc_ev);
@@ -863,7 +920,9 @@ srl_is_window_event (SRLEvent *event)
 	    event->type == SRL_EVENT_WINDOW_DEACTIVATE ||
 	    event->type == SRL_EVENT_WINDOW_RENAME ||
 	    event->type == SRL_EVENT_TAB_ADDED ||
+	    event->type == SRL_EVENT_CHILDREN_ADDED ||
 	    event->type == SRL_EVENT_CONTEXT_SWITCHED ||
+	    event->type == SRL_EVENT_CHILDREN_REMOVED ||
 	    event->type == SRL_EVENT_TAB_REMOVED;
 }
 static gboolean
@@ -940,13 +999,6 @@ srl_is_tooltip_event (SRLEvent *event)
     return  event->type == SRL_EVENT_TOOLTIP_SHOW;
 }
 
-static gboolean
-srl_is_mouse_event (SRLEvent *event)
-{
-    srl_assert (event);
-    
-    return  event->type == SRL_EVENT_MOUSE_ABS;
-}
 
 static SRLEventPriority
 srle_get_priority (SRLEvent *event)
@@ -957,8 +1009,6 @@ srle_get_priority (SRLEvent *event)
 	return SRL_EVENT_PRIORITY_WATCHED;
     if (srl_is_window_event (event))
 	return SRL_EVENT_PRIORITY_WINDOW;
-    if (srl_is_mouse_event (event))
-	return SRL_EVENT_PRIORITY_MOUSE;
     if (srl_is_tooltip_event (event))
 	return SRL_EVENT_PRIORITY_TOOLTIP;
     return SRL_EVENT_PRIORITY_FOCUS;
@@ -1146,30 +1196,6 @@ srl_notify_clients_obj (SRLEvent *event,
     return obj != NULL;
 }
 
-static gboolean
-srl_notify_clients_mouse (SRLEvent *event,
-			  SREventType type)
-{
-    SRPoint *point;
-    SREvent *ev;
-
-    srl_assert (event);
-
-    ev = sre_new ();
-    point = g_new0 (SRPoint, 1);
-
-    ev->type = type;
-    ev->data = point;
-    point->x = event->acc_ev->detail1;
-    point->y = event->acc_ev->detail2;
-    ev->data_destructor = (SREventDataDestructor) g_free;
-    srl_log_gnopernicus_event_user_mouse (event, ev);
-    srl_notify_all_clients (ev, 0);
-    sre_release_reference (ev);
-
-    return TRUE;
-}
-
 static gboolean
 srl_report_event_from_lasts_to_clients (gint index,
 				        SREventType type,
@@ -1214,18 +1240,6 @@ srl_report_obj_event_to_clients (gint index,
     return TRUE;
 }
 
-static gboolean
-srl_report_mouse_event_to_clients (gint index,
-				   SREventType type)
-{
-    srl_assert (0 <= index && index < SRL_EVENT_PRIORITY_LAST);
-
-    srl_report_event_from_lasts_to_clients (index, type,
-						srl_notify_clients_mouse);
-
-    return TRUE;
-}
-
 
 static gboolean
 srl_report_event_to_clients (gpointer data)
@@ -1238,8 +1252,6 @@ srl_report_event_to_clients (gpointer data)
 					    SR_EVENT_SRO);
     srl_report_obj_event_to_clients (SRL_EVENT_PRIORITY_TOOLTIP,
 					    SR_EVENT_TOOLTIP);
-    srl_report_mouse_event_to_clients (SRL_EVENT_PRIORITY_MOUSE,
-					    SR_EVENT_MOUSE);
 
     srl_idle_installed = srl_idle_need;
     srl_idle_need = FALSE;
@@ -1617,9 +1629,15 @@ srle_change_type (SRLEvent *event)
     }
     
     if (srle_has_type (event, SRL_EVENT_CHILDREN_ADDED))
-	event->type = SRL_EVENT_TAB_ADDED;
+    {
+	if (srle_acc_has_role (event, SPI_ROLE_PAGE_TAB))
+	    event->type = SRL_EVENT_TAB_ADDED;
+    }
     else if (srle_has_type (event, SRL_EVENT_CHILDREN_REMOVED))
-	event->type = SRL_EVENT_TAB_REMOVED;
+    {
+	if (srle_acc_has_role (event, SPI_ROLE_PAGE_TAB))
+	    event->type = SRL_EVENT_TAB_REMOVED;
+    }
 	
     /* Convert the "state-changed:selected" event for a _selected_ page-tab
        into a "focus:" event */
@@ -1683,6 +1701,42 @@ srl_set_checked_info (Accessible *acc,
     return TRUE;
 }
 
+
+AccessibleText*
+srl_get_acc_text_from_children (Accessible *acc)
+{
+    gint i, cnt;
+    AccessibleText *rv = NULL;
+
+    srl_assert (acc);
+
+    cnt = Accessible_getChildCount (acc);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child = Accessible_getChildAtIndex (acc, i);
+	if (!child)
+	    continue;
+	if (Accessible_getRole (child) == SPI_ROLE_TEXT)
+	    rv = Accessible_getText (child);
+	Accessible_unref (child);
+	if (rv)
+	    return rv;
+    }
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child = Accessible_getChildAtIndex (acc, i);
+	if (!child)
+	    continue;
+	rv = srl_get_acc_text_from_children (child);
+	Accessible_unref (child);
+	if (rv)
+	    return rv;
+    }
+    
+
+    return NULL;
+}
+
 static gboolean
 srl_set_info (Accessible *acc,
 	      SRLLastInfo *info)
@@ -1700,6 +1754,8 @@ srl_set_info (Accessible *acc,
     }
 
     text = Accessible_getText (acc);
+    if (!text && Accessible_getRole (acc) == SPI_ROLE_COMBO_BOX)
+    	text = srl_get_acc_text_from_children (acc);
     if (text)
     {
         srl_set_text_info (text, info);
@@ -1866,6 +1922,30 @@ srl_selection_event_is_reported (SRLEvent *event)
 static SRLLastInfo last_info_focus, last_info_watched;
 static gboolean srl_set_last_focus (Accessible *acc);
 
+static gboolean
+srle_is_parent_of_last_focus (SRLEvent *event)
+{
+    Accessible *parent;
+
+    if (!srl_last_focus)
+	return FALSE;
+
+    parent = srl_last_focus;
+    Accessible_ref (parent);
+    
+    while (parent && parent != srle_get_acc (event))
+    {
+	Accessible *tmp = parent;
+	parent = Accessible_getParent (parent);
+	Accessible_unref (tmp);
+    }
+    if (parent)
+    	Accessible_unref (parent);
+    
+    return parent != NULL;
+}
+
+
 static gboolean
 srl_event_is_reported (SRLEvent *event)
 {
@@ -1900,8 +1980,7 @@ srl_event_is_reported (SRLEvent *event)
 	    }
 	}
     }
-    else if (srl_is_mouse_event (event)	||
-	srl_is_tooltip_event (event))
+    else if (srl_is_tooltip_event (event))
 	rv = TRUE;
     else if (srle_has_type (event, SRL_EVENT_FOCUS))
 	rv = srle_acc_has_state (event, SPI_STATE_FOCUSED) ||
@@ -2014,8 +2093,13 @@ srl_event_is_reported (SRLEvent *event)
     }
     if ((srle_has_type (event, SRL_EVENT_TAB_ADDED) || 
          srle_has_type (event, SRL_EVENT_TAB_REMOVED))) 
-	rv = srle_acc_has_role (event, SPI_ROLE_PAGE_TAB);
-	
+	rv = srle_acc_has_role (event, SPI_ROLE_PAGE_TAB) &&
+		!srle_is_parent_of_last_focus (event);
+
+    if ((srle_has_type (event, SRL_EVENT_CHILDREN_ADDED) || 
+         srle_has_type (event, SRL_EVENT_CHILDREN_REMOVED))) 
+	rv = srle_acc_has_role (event, SPI_ROLE_PARAGRAPH);
+
     /*In some situations we have two 'focus:' events for a page-tab 
      (an "object:state-changed:selected" event which was converted into
       a focus event _and_ a "focus:" event) => report a single "focus:" event*/	
@@ -2308,58 +2392,6 @@ srl_remove_client (SRLClientHandle client)
 }
 
 
-gboolean
-srl_mouse_move (gint x,
-		gint y)
-{
-    return SPI_generateMouseEvent (x, y, "abs") ? TRUE : FALSE;
-}		
-
-gboolean
-srl_mouse_click (gint button)
-{
-    gchar action[] = "b1c";
-    
-    if (button == SR_MOUSE_BUTTON_LEFT)
-	action[1] = '1';
-    else if (button == SR_MOUSE_BUTTON_RIGHT)
-	action[1] = '2';
-    else
-	srl_assert_not_reached ();
-    
-    return SPI_generateMouseEvent (-1, -1, action) ? TRUE : FALSE;
-}
-
-gboolean
-srl_mouse_button_down (gint button)
-{
-    gchar action[] = "b1p";
-    
-    if (button == SR_MOUSE_BUTTON_LEFT)
-	action[1] = '1';
-    else if (button == SR_MOUSE_BUTTON_RIGHT)
-	action[1] = '2';
-    else
-	srl_assert_not_reached ();
-    return SPI_generateMouseEvent (-1,-1 , action) ? TRUE : FALSE;
-}		
-
-gboolean
-srl_mouse_button_up (gint button)
-{
-    gchar action[] = "b1r";
-    
-    if (button == SR_MOUSE_BUTTON_LEFT)
-	action[1] = '1';
-    else if (button == SR_MOUSE_BUTTON_RIGHT)
-	action[1] = '2';
-    else
-	srl_assert_not_reached ();
-    
-    return SPI_generateMouseEvent (-1,-1 , action) ? TRUE : FALSE;
-}		
-
-
 gboolean
 srl_set_watch_for_object (SRObject *obj)
 {
@@ -2401,38 +2433,6 @@ srl_is_object_focused (SRObject *obj)
     return sro_get_acc (obj) == srl_last_focus || sro_get_acc (obj) == srl_last_focus2;
 }
 
-static Accessible*
-srl_get_focused_from_acc (Accessible *acc)
-{
-    Accessible *focused = NULL;
-    gint cnt, i;
-
-    srl_assert (acc);
-    
-    cnt = Accessible_getChildCount (acc);
-    for (i = 0; i < cnt; i++)
-    {
-	Accessible *child;
-	child = Accessible_getChildAtIndex (acc, i);
-	if (!child)
-	    continue;
-	if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
-	{
-	    focused = child;
-	    Accessible_ref (focused);
-	}
-	else
-	    focused = srl_get_focused_from_acc (child);
-	
-	Accessible_unref (child);
-	if (focused)
-	    break;
-    }
-
-    return focused;
-}
-
-
 static Accessible*
 srl_get_focused_from_active_toplevel (Accessible *window)
 {
